from django import template

register = template.Library()

"""
This tag is used to give some html as argument to an {% include %} tag.

For instance:

{% htmlparams %}
    {% htmlparam content %}
        <span>click me</span>
    {% endhtmlparam %}
    {% include "website/components/button.html"%}
{% endhtmlparams %}

you have to wrap {% include %} within {% htmlparams %} then you can add {% htmlparam VARIABLE_NAME %} with some html within and use the variable within the component.

Included button:

<button>
    {{ content }}
</button>

"""


class HtmlParamNode(template.Node):
    def __init__(self, var_name, nodelist):
        self.var_name = var_name
        self.nodelist = nodelist

    def render(self, context):
        # do not render anything but stores the output in the context
        output = self.nodelist.render(context)
        context[self.var_name] = output
        return ""


class HtmlParamsNode(template.Node):
    def __init__(self, nodelist):
        self.nodelist = template.NodeList()
        self.paramnodelist = template.NodeList()
        for node in nodelist:
            if isinstance(node, HtmlParamNode):
                self.paramnodelist.append(node)
            else:
                self.nodelist.append(node)

    def render(self, context):
        with context.push():
            ## Render the htmlparam nodes to update the context
            self.paramnodelist.render(context)
            output = self.nodelist.render(context)
        return output


@register.tag(name="htmlparam")
def do_htmlparam(parser, token):
    try:
        tag_name, var_name = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError(
            "%r tag requires a single argument" % token.contents.split()[0]
        )
    nodelist = parser.parse(("endhtmlparam",))
    parser.delete_first_token()
    return HtmlParamNode(var_name, nodelist)


@register.tag(name="htmlparams")
def do_htmlparams(parser, token):
    nodelist = parser.parse(("endhtmlparams",))
    parser.delete_first_token()
    return HtmlParamsNode(nodelist)



# Generated by Django 5.2.3 on 2025-06-19 12:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('indicators_lib', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='filter',
            name='color',
            field=models.TextField(blank=True, help_text='format hexadécimale : #FF11DD', null=True, verbose_name='Couleur'),
        ),
        migrations.AddField(
            model_name='indicator',
            name='min_mesh',
            field=models.TextField(choices=[('fr', 'National'), ('reg', 'Region'), ('dep', 'Department'), ('epci', 'Epci'), ('com', 'Town')], default='com'),
        ),
        migrations.AlterField(
            model_name='indicator',
            name='show_evolution',
            field=models.BooleanField(default=True),
        ),
    ]



from django.apps import apps

from territories_dashboard_lib.indicators_lib.query.utils import run_custom_query


def refresh_filters(dimension):
    if dimension:
        query = f'SELECT DISTINCT({dimension.db_name}) as filter FROM "{dimension.indicator.db_table_prefix}_reg"'
        results = run_custom_query(query)
        Filter = apps.get_model("indicators_lib", "Filter")
        Filter.objects.filter(dimension=dimension).delete()
        Filter.objects.bulk_create(
            [
                Filter(dimension=dimension, db_name=value["filter"], order=index)
                for index, value in enumerate(results)
            ]
        )



from django.db import models


class CommonModel(models.Model):
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True



# Generated by Django 5.1.6 on 2025-06-30 12:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tracking_lib', '0002_alter_event_created_at_alter_page_created_at'),
    ]

    operations = [
        migrations.CreateModel(
            name='CookieInfo',
            fields=[
                ('id', models.TextField(primary_key=True, serialize=False)),
                ('country', models.TextField(blank=True, null=True)),
            ],
        ),
    ]



from django.shortcuts import get_object_or_404, redirect, render
from django.urls import reverse
from django.views.decorators.gzip import gzip_page

from territories_dashboard_lib.indicators_lib.models import Indicator, Theme
from territories_dashboard_lib.superset_lib.logic import make_filter
from territories_dashboard_lib.superset_lib.models import Dashboard
from territories_dashboard_lib.superset_lib.serializers import serialize_dashboard
from territories_dashboard_lib.tracking_lib.logic import track_page
from territories_dashboard_lib.website_lib.models import (
    GlossaryItem,
    LandingPage,
    StaticPage,
)
from territories_dashboard_lib.website_lib.params import with_params
from territories_dashboard_lib.website_lib.serializers import (
    serialize_indicator,
    serialize_sub_themes,
)


@gzip_page
def landing_page_view(request):
    landing_page = LandingPage.objects.first()
    response = render(
        request,
        "territories_dashboard_lib/website/pages/page.html",
        {"landing_page": landing_page},
    )
    response = track_page(request=request, response=response)
    return response


@gzip_page
def static_page_view(request, page_url):
    static_page = get_object_or_404(StaticPage, url=page_url)
    return render(
        request,
        "territories_dashboard_lib/website/pages/static/page.html",
        {"static_page": static_page},
    )


@gzip_page
def lexique_page_view(request):
    lexique = GlossaryItem.objects.all()
    context = {"lexique": lexique}
    response = render(
        request, "territories_dashboard_lib/website/pages/lexique/page.html", context
    )
    return response


def themes_redirect_view(request):
    first_theme = Theme.objects.order_by("ordering").first()
    target_url = reverse("website:theme", kwargs={"theme_name": first_theme.name})
    get_params = request.GET.urlencode()
    if get_params:
        target_url = f"{target_url}?{get_params}"
    return redirect(target_url)


@gzip_page
@with_params
def comparison_view(request, *, theme_name, context):
    theme = get_object_or_404(Theme, name=theme_name)
    sub_themes = serialize_sub_themes(theme)
    themes = Theme.objects.order_by("ordering")
    context["theme"] = theme
    context["themes"] = themes
    context["sub_themes"] = sub_themes
    response = render(
        request,
        "territories_dashboard_lib/website/pages/indicators/comparaison/[theme]/page.html",
        context,
    )
    response = track_page(
        request=request,
        response=response,
        params=context["params"],
        theme=theme,
    )
    return response


def comparison_redirect_view(request):
    first_theme = Theme.objects.order_by("ordering").first()
    target_url = reverse("website:comparison", kwargs={"theme_name": first_theme.name})
    get_params = request.GET.urlencode()
    if get_params:
        target_url = f"{target_url}?{get_params}"
    return redirect(target_url)


@gzip_page
@with_params
def theme_page_view(request, *, theme_name, context):
    theme = get_object_or_404(Theme, name=theme_name)
    sub_themes = serialize_sub_themes(theme)
    themes = Theme.objects.all().order_by("ordering")
    context["theme"] = theme
    context["themes"] = themes
    context["sub_themes"] = sub_themes
    response = render(
        request,
        "territories_dashboard_lib/website/pages/indicators/themes/page.html",
        context,
    )
    response = track_page(
        request=request,
        response=response,
        params=context["params"],
        theme=theme,
    )
    return response


def apply_filters(request, indicator):
    filters = []
    for dimension in indicator.dimensions.all():
        url_filters = request.GET.getlist(dimension.db_name)
        db_filters = [f for f in dimension.filters.all()]
        for f in db_filters:
            f.set_from_url = "!" + f.db_name in url_filters or f.db_name in url_filters
            f.current = (
                f.default
                and "!" + f.db_name not in url_filters
                or f.db_name in url_filters
            )
        filters.append([dimension, db_filters])
    return filters


@gzip_page
@with_params
def indicator_details_view(request, *, indicator_name, context):
    indicator = get_object_or_404(Indicator, name=indicator_name)
    theme = indicator.sub_theme.theme
    sub_themes = serialize_sub_themes(theme)
    themes = Theme.objects.all().order_by("ordering")
    filters = apply_filters(request, indicator)
    context["indicator"] = serialize_indicator(indicator)
    context["themes"] = themes
    context["theme"] = theme
    context["subtheme"] = indicator.sub_theme
    context["sub_themes"] = sub_themes
    context["filters"] = filters
    response = render(
        request,
        "territories_dashboard_lib/website/pages/indicators/details/page.html",
        context=context,
    )
    response = track_page(
        request=request,
        response=response,
        params=context["params"],
        indicator=indicator,
        theme=theme,
    )
    return response


@with_params
def superset_view(request, dashboard_name, context):
    dashboard = get_object_or_404(Dashboard, short_name=dashboard_name)
    context["dashboard"] = serialize_dashboard(dashboard)
    filter_string = make_filter(
        dashboard,
        context["params"]["territory_id"],
        context["params"]["territory_mesh"],
    )
    if filter_string:
        context["dashboard_filter"] = filter_string
    response = render(
        request,
        "territories_dashboard_lib/website/pages/superset/page.html",
        context=context,
    )
    response = track_page(request=request, response=response)
    return response


def handler_404_view(request, exception):
    return render(request, "territories_dashboard_lib/website/404.html", status=404)


def handler_500_view(request):
    return render(request, "territories_dashboard_lib/website/500.html", status=500)


@gzip_page
def sitemap_view(request):
    context = {
        "themes": [
            {"title": theme.title, "name": theme.name}
            for theme in Theme.objects.all().order_by("ordering")
        ],
        "indicators": [
            {"title": indicator.title, "name": indicator.name}
            for indicator in Indicator.objects.all().order_by("index_in_theme")
        ],
        "static_pages": [
            {"title": static_page.name, "url": static_page.url}
            for static_page in StaticPage.objects.all()
        ],
        "lexique": GlossaryItem.objects.exists(),
    }
    response = render(
        request, "territories_dashboard_lib/website/pages/sitemap/page.html", context
    )
    response = track_page(request=request, response=response)
    return response



from django.conf import settings
from django.db import connections


def get_breakdown_dimension(indicator):
    breakdown_dimension = (
        indicator.dimensions.filter(is_breakdown=True).first()
        if indicator.dimensions.count() > 1
        else indicator.dimensions.first()
    )
    return breakdown_dimension


def run_custom_query(query, params=None):
    with connections[settings.INDICATORS_DATABASE].cursor() as cursor:
        cursor.execute(query, params)
        columns = [col[0] for col in cursor.description]
        rows = cursor.fetchall()
        results = [dict(zip(columns, row)) for row in rows]
        return results



from django.urls import path

from .views import (
    geo_features_view,
    main_territory_view,
    precise_view,
    search_territories_view,
    territories_view,
)

app_name = "geo-api"

urlpatterns = [
    path(
        "geo-features/",
        geo_features_view,
        name="geo-features",
    ),
    path(
        "main-territory/",
        main_territory_view,
        name="main-territory",
    ),
    path("precise/", precise_view, name="precise"),
    path("territories/", territories_view, name="territories"),
    path("search-territories/", search_territories_view, name="search"),
]



from .details import get_values_for_submesh_territories


def get_territories_histogram_data(territory_values, unite):
    if all([t["valeur"] is None for t in territory_values]):
        return ({}, [])
    values_for_min_max = [
        data["valeur"] for data in territory_values if data["valeur"] is not None
    ]
    min_value_last_year = min(values_for_min_max) if values_for_min_max else 0
    max_value_last_year = max(values_for_min_max) if values_for_min_max else 0

    # if we are between 99.5 and 100.5, spread histogram for better visualization
    if 99.5 < min_value_last_year < 100.5 and 99.5 < max_value_last_year < 100.5:
        min_value_last_year = 90

    histogram_values_dict = {}
    for territory_data in territory_values:
        territory_id = territory_data["geocode"]
        if territory_id not in histogram_values_dict:
            histogram_values_dict[territory_id] = {
                "valeur": 0,
                "geoname": territory_data["geoname"],
            }
        histogram_values_dict[territory_id]["valeur"] += (
            territory_data["valeur"] if territory_data["valeur"] is not None else 0
        )
    histogram_values = list(histogram_values_dict.values())

    deciles = [
        min_value_last_year + (max_value_last_year - min_value_last_year) * (i / 10)
        for i in range(10 if min_value_last_year != max_value_last_year else 1)
    ]

    data_by_decile = [
        {
            "decile": decile,
            "count": len(
                [
                    data
                    for data in histogram_values
                    if decile
                    <= data["valeur"]
                    <= (deciles[i + 1] if i < len(deciles) - 1 else max_value_last_year)
                ]
            ),
            "text": "\n".join(
                [
                    data["geoname"]
                    for data in histogram_values
                    if decile
                    <= data["valeur"]
                    <= (deciles[i + 1] if i < len(deciles) - 1 else max_value_last_year)
                ]
            ),
        }
        for i, decile in enumerate(deciles)
    ]

    datasets_histogram_bar_chart = {
        "label": unite,
        "data": [{"x": data["decile"], "y": data["count"]} for data in data_by_decile],
        "comments": [data["text"] for data in data_by_decile],
        "backgroundColor": "#6a6af4",
    }

    return (datasets_histogram_bar_chart, deciles)


def get_indicator_histogram_data(indicator, territory, submesh, filters):
    indicator_details = {}
    territory_values = get_values_for_submesh_territories(
        indicator, submesh, territory, filters
    )
    datasets_histogram_bar_chart, deciles = get_territories_histogram_data(
        territory_values, indicator.unite
    )

    indicator_details["datasetsHistogramBarChart"] = datasets_histogram_bar_chart
    indicator_details["deciles"] = deciles

    return indicator_details



from django import forms
from django.contrib import admin
from django.db import models
from django.utils.safestring import mark_safe

from territories_dashboard_lib.geo_lib.models import GeoElement, GeoFeature


class GeoColumnInLine(admin.TabularInline):
    model = GeoElement
    fields = ["name", "label", "filterable"]
    extra = 0
    formfield_overrides = {
        models.TextField: {"widget": forms.TextInput()},
    }


class GeoFeatureForm(forms.ModelForm):
    svg_file = forms.FileField(required=False, label="Charger un fichier SVG")

    class Meta:
        model = GeoFeature
        fields = [
            "indicator",
            "name",
            "title",
            "unite",
            "geo_type",
            "color",
            "show_on_fr_level",
            "point_icon_svg",
            "svg_file",
        ]

    def clean(self):
        cleaned_data = super().clean()
        svg_file = cleaned_data.get("svg_file")

        if svg_file:
            svg_content = svg_file.read().decode("utf-8")
            cleaned_data["point_icon_svg"] = svg_content

        return cleaned_data


class GeoTableAdmin(admin.ModelAdmin):
    form = GeoFeatureForm
    inlines = [GeoColumnInLine]
    formfield_overrides = {
        models.TextField: {"widget": forms.TextInput()},
    }
    list_display = ["__str__", "indicator"]
    list_filter = ["indicator"]
    readonly_fields = ["svg_preview"]

    def svg_preview(self, obj):
        if obj.point_icon_svg:
            return mark_safe(obj.point_icon_svg)
        return "-"

    svg_preview.short_description = "Prévisualisation du SVG"


admin.site.register(GeoFeature, GeoTableAdmin)



from django import template
from django.urls import reverse
from territories_dashboard_lib.indicators_lib.enums import get_all_meshes

register = template.Library()


@register.filter
def get_item(dictionary, key):
    return dictionary.get(key)


@register.filter
def indicator_api_urls(indicator_dict):
    view_names = [
        "download-indicator-methodo",
        "statistics",
        "values",
        "histogram",
        "top-10",
        "details-table",
        "details-table-export",
        "comparison-histogram",
        "proportions",
    ]
    return {
        view_name: reverse(
            f"indicators-api:{view_name}", kwargs={"name": indicator_dict["name"]}
        )
        for view_name in view_names
    }


@register.filter
def should_mesh_analysis(params, indicator):
    meshes = get_all_meshes()
    above_min_mesh = meshes.index(params["mesh"]) <= meshes.index(indicator["min_mesh"])
    return above_min_mesh



# Generated by Django 5.2.3 on 2025-06-18 13:03

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Dashboard',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('superset_id', models.TextField(help_text="Un utilisateur administrateur de l'instance Superset doit accéder aux paramètres du dashboard et cliquer sur 'embed dashboard' pour récupérer cet ID nécessaire à la connexion avec l'instance Superset.", unique=True, verbose_name='Embed ID de Superset')),
                ('short_name', models.TextField(help_text="Pour l'URL, ne mettre que des lettres minuscules sans accents et des tirets", unique=True, verbose_name='Nom court')),
                ('label', models.TextField(help_text='Nom du dashboard à afficher dans la liste.', unique=True, verbose_name='Label')),
                ('order', models.IntegerField(default=1, help_text="Numéro d'ordre dans la dropdown de sélection, les dashboards sont triés du plus petit numéro au plus grand.", verbose_name="Numéro d'ordre")),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Filter',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('superset_id', models.TextField(blank=True, help_text="ID du native filter du territoire, qui permettra d'initialiser les filtres du dasbhoard au territoire sélectionné dans l'application. Pour le récupérer c'est un petit parcours du combattant : aller sur le dashboard dans superset, cliquer sur modifier le dashboard, cliquer sur les trois petits points, puis sur modifier les propriétés. Cliquer sur avancé, copier le json et le coller dans un site comme : https://jsonformatter.curiousconcept.com/ pour mieux le voir. Chercher 'global_chart_configuration' puis 'native_filter_configuration'. Chercher le native filter lié au choix du territoire. Copier l'id qui dans son nom, le nom est de la forme NATIVE_FILTER-ID. Promis, c'est le plus simple que j'ai trouvé !", null=True)),
                ('superset_col', models.TextField(blank=True, help_text="Nom de la colonne en base de données liée au filtre sur le territoire. Pour trouver le nom faire les mêmes étapes que pour geo_filter_id et chercher 'column' dans les paramètres json du native filter.", null=True)),
                ('mesh', models.TextField(blank=True, choices=[('fr', 'National'), ('reg', 'Region'), ('dep', 'Department'), ('epci', 'Epci'), ('com', 'Town')], help_text="Maille du territoire sur lequel s'effectue le filtre.", null=True)),
                ('dashboard', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='filters', to='superset_lib.dashboard')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from territories_dashboard_lib.indicators_lib.enums import MeshLevel
from territories_dashboard_lib.indicators_lib.query.utils import run_custom_query

from .models import Dashboard


def get_territory_meshes(territory_id: str, territory_mesh: MeshLevel):
    current_reg = None
    current_dep = None
    current_epci = None
    current_com = None
    if territory_mesh != MeshLevel.National:
        mapped_mesh = "DEPCOM" if territory_mesh == "com" else territory_mesh.upper()
        query = f"""
        SELECT
            "NOM_REG" as reg_name,
            "NOM_DEP" as dep_name,
            "NOM_EPCI" as epci_name,
            "NOM_DEPCOM" as com_name
        FROM arborescence_geo
        WHERE "{mapped_mesh}" = '{territory_id}'
        LIMIT 1
        """
        row = run_custom_query(query)[0]
        if territory_mesh in [
            MeshLevel.Region,
            MeshLevel.Department,
            MeshLevel.Epci,
            MeshLevel.Town,
        ]:
            current_reg = row["reg_name"]
        if territory_mesh in [
            MeshLevel.Department,
            MeshLevel.Epci,
            MeshLevel.Town,
        ]:
            current_dep = row["dep_name"]
        if territory_mesh in [
            MeshLevel.Epci,
            MeshLevel.Town,
        ]:
            current_epci = row["epci_name"]
        if territory_mesh in [
            MeshLevel.Town,
        ]:
            current_com = row["com_name"]
    territory_meshes = {
        MeshLevel.Region: current_reg,
        MeshLevel.Department: current_dep,
        MeshLevel.Epci: current_epci,
        MeshLevel.Town: current_com,
    }
    return territory_meshes


def make_filter(dashboard: Dashboard, territory_id: str, territory_mesh: MeshLevel):
    if dashboard.filters.count() == 0:
        return None
    territory_meshes = get_territory_meshes(territory_id, territory_mesh)
    filters = []
    for f in dashboard.filters.all():
        value = territory_meshes.get(f.mesh)
        if value:
            value = value.replace("'", "!'")  # RISON escape
            filter_string = f"""NATIVE_FILTER-{f.superset_id}:(__cache:(label:'{value}',validateStatus:!f,value:!('{value}')),extraFormData:(filters:!((col:{f.superset_col},op:IN,val:!('{value}')))),filterState:(label:'{value}',validateStatus:!f,value:!('{value}')),id:NATIVE_FILTER-{f.superset_id},ownState:())"""
            filters.append(filter_string)
    if not filters:
        return None
    return f"""({",".join(filters)})"""



import http.client
import json

from django.conf import settings


def get_guest_token(dashboard_id):
    """
    Function copied from https://snum.gitlab-pages.din.developpement-durable.gouv.fr/ds/gd3ia/offre-dataviz-documentation/05-Documentation_SUPERSET_INTEGRATION/
    """
    conn = http.client.HTTPSConnection(settings.SUPERSET_DOMAIN)
    params = json.dumps(
        {
            "provider": "db",
            "refresh": "True",
            "username": settings.SUPERSET_USERNAME,
            "password": settings.SUPERSET_PASSWORD,
        }
    )
    conn.request(
        "POST", "/api/v1/security/login", params, {"Content-Type": "application/json"}
    )
    response = conn.getresponse()
    content = json.loads(response.read())
    access_token = content["access_token"]

    # Recuperation du token CSRF et cookie de session associe
    headers = {"Authorization": f"Bearer {access_token}"}
    conn.request("GET", "/api/v1/security/csrf_token/", None, headers)
    response = conn.getresponse()
    content = json.loads(response.read())
    csrf_token = content["result"]
    cookie = response.headers["set-cookie"].split("; ")[0]

    # Recuperation du guest_token pour l'affichage du diagramme
    params = json.dumps(
        {
            "resources": [{"id": dashboard_id, "type": "dashboard"}],
            "rls": [
                {
                    # Clause SQL appliquée à la récupération des donnes du dashboard
                    # Elle peut être ajustée pour limiter les données affichées
                    # dans le dashboard en fonction du profil utilisateur
                    # La valeur "1=1" permet de n'appliquer aucun filtre
                    "clause": "1=1"
                }
            ],
            "user": {
                "first_name": "Prenom",
                "last_name": "Nom",
                "username": settings.SUPERSET_USERNAME,
            },
        }
    )
    headers = {
        "Content-Type": "application/json",
        "Authorization": f"Bearer {access_token}",
        "X-Csrftoken": csrf_token,
        "Cookie": cookie,
    }
    conn.request("POST", "/api/v1/security/guest_token/", params, headers)
    response = conn.getresponse()
    guest_token = json.loads(response.read())["token"]
    return guest_token



import base64
import datetime
import gzip
import json
from typing import Optional

from django.http import HttpResponse, JsonResponse
from django.shortcuts import get_object_or_404
from django.views.decorators.cache import cache_control
from django.views.decorators.http import require_GET
from psycopg2.sql import SQL, Identifier, Literal
from pydantic import BaseModel

from territories_dashboard_lib.indicators_lib.enums import (
    FRANCE_GEOLEVEL_TITLES,
    MESH_DB,
    GeoLevel,
    MeshLevel,
)
from territories_dashboard_lib.indicators_lib.query.commons import get_territories_ids
from territories_dashboard_lib.indicators_lib.query.utils import run_custom_query

from .enums import GeoFeatureType
from .models import GeoFeature


class GeoFeaturesParams(BaseModel):
    mesh: MeshLevel
    geo_level: GeoLevel
    main_territories: str
    last: Optional[str] = None
    limit: Optional[str] = "1000"
    feature: str


class DateTimeEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, (datetime.date, datetime.datetime)):
            return (
                obj.date().isoformat()
                if isinstance(obj, datetime.datetime)
                else obj.isoformat()
            )
        return super().default(obj)


@require_GET
@cache_control(max_age=3600)
def geo_features_view(request):
    params = GeoFeaturesParams(**request.GET.dict())

    geo_feature = get_object_or_404(GeoFeature, id=params.feature)
    geo_level = params.geo_level
    mesh = params.mesh
    main_territory_codes = params.main_territories.split(",")
    last = params.last
    limit = params.limit

    last_year_query = (
        f"SELECT DISTINCT annee FROM {geo_feature.name} ORDER BY annee DESC"
    )
    years = run_custom_query(last_year_query)
    last_year = years[0].get("annee")

    territories_ids = get_territories_ids(main_territory_codes, geo_level, mesh)

    col_id_name = f"code_{mesh}"
    columns = [el.name for el in geo_feature.items.all()]

    coma = ", " if columns else ""
    select_geo_code = (
        f", geo.{col_id_name} AS geo_code"
        if geo_feature.geo_type == GeoFeatureType.point
        else ""
    )

    if geo_feature.geo_type == GeoFeatureType.point:
        where_clause = f"WHERE geo.{col_id_name} IN ('{"', '".join(territories_ids)}')"
    else:
        where_clause = (
            f"JOIN contours_simplified_{geo_level} contours ON ST_intersects(geo.geometry, contours.geometry)\n"
            f"WHERE contours.code IN ('{"', '".join(main_territory_codes)}')"
        )
    where_clause += f" AND annee = {last_year}"

    pagination = f"AND order_id > {last or 0} ORDER BY order_id ASC LIMIT {limit}"

    query = (
        f"SELECT ST_asgeojson(geo.geometry) AS geojson, order_id{coma + ', '.join(columns)}\n"
        f"{select_geo_code}\n"
        f"FROM {geo_feature.name} AS geo\n"
        f"{where_clause} {pagination}"
    )

    # Execute the query
    results = run_custom_query(query)
    items = geo_feature.items.all()
    # Transform results
    data = [
        {
            "type": "Feature",
            "id": r.get("geo_code"),
            "properties": {el.label: r.get(el.name) for el in items},
            "geometry": json.loads(r.get("geojson")),
        }
        for r in results
    ]

    last_queried_order_id = results[-1]["order_id"] if results else None

    compressed_data = gzip.compress(json.dumps(data, cls=DateTimeEncoder).encode())
    compressed_base64 = base64.b64encode(compressed_data).decode()

    return JsonResponse({"data": compressed_base64, "last": last_queried_order_id})


class MainTerritoryParams(BaseModel):
    geo_level: GeoLevel
    geo_id: str


@require_GET
@cache_control(max_age=3600)
def main_territory_view(request):
    params = MainTerritoryParams(**request.GET.dict())

    codes = ", ".join(f"'{code.strip()}'" for code in params.geo_id.split(","))

    query = f"""
        SELECT code as id, ST_asgeojson(ST_simplify(geometry, 0.01)) AS polygon
        FROM contours_simplified_{params.geo_level}
        WHERE code IN ({codes})
        """

    results = run_custom_query(query)

    data = [
        {
            "id": r.get("id"),
            "polygon": json.loads(r.get("polygon")),
        }
        for r in results
    ]

    return JsonResponse(data, safe=False)


@require_GET
@cache_control(max_age=3600)
def precise_view(request):
    territories_ids = request.GET.get("territories", "").split(",")
    mesh_level = request.GET.get("mesh")

    if not mesh_level:
        return JsonResponse({"error": "Missing 'mesh' parameter"}, status=400)

    mapped_mesh_level = "DEPCOM" if mesh_level == "com" else mesh_level.upper()

    query = f"""
        SELECT "{mapped_mesh_level}" AS id, ST_asgeojson(geometry) AS polygon
        FROM contours_geo_ign_{mesh_level}
        WHERE "{mapped_mesh_level}" IN ('{"', '".join(territories_ids)}')
        """

    results = run_custom_query(query)

    territories = {r.get("id"): json.loads(r.get("polygon")) for r in results}

    return JsonResponse(territories)


class TerritoryFeatureParams(BaseModel):
    mesh: MeshLevel
    geo_level: GeoLevel | None = None
    main_territories: str | None = None
    codes: str | None = None


@require_GET
@cache_control(max_age=3600)
def territories_view(request):
    params = TerritoryFeatureParams(**request.GET.dict())
    geo_level = params.geo_level
    mesh = params.mesh

    if params.codes:
        territories_ids = params.codes.split(",")
    else:
        main_territory_codes = params.main_territories.split(",")
        territories_ids = get_territories_ids(main_territory_codes, geo_level, mesh)

    query = f"""
        SELECT code AS id, ST_asgeojson(geometry) AS polygon
        FROM contours_simplified_{params.mesh}
        WHERE code IN ('{"', '".join(territories_ids)}')
        """

    results = run_custom_query(query)

    data = [
        {"type": "Feature", "id": r.get("id"), "geometry": json.loads(r.get("polygon"))}
        for r in results
    ]
    return JsonResponse(data, safe=False)


def _fill_territory_li(code, name, mesh):
    label = name if mesh == MeshLevel.National else f"{name} - {code}"
    return f"""<li data-code="{code}" data-name="{name}" onclick="chooseTerritory(this)"><button aria-label="Choisir {label}">{label}</button></li>"""


@require_GET
@cache_control(max_age=3600)
def search_territories_view(request):
    request_mesh = request.GET.get("mesh")
    if request_mesh in MeshLevel:
        mesh = request_mesh
    else:
        raise ValueError("mesh")
    if mesh == MeshLevel.National:
        lis = []
        for code, name in FRANCE_GEOLEVEL_TITLES.items():
            li = _fill_territory_li(code, name, mesh)
            lis.append(li)
        return HttpResponse("\n".join(lis))
    mesh_db = MESH_DB[mesh]
    pagination = 20
    search = request.GET.get("search", "")
    offset = int(request.GET.get("offset", 0))
    query = SQL("""
    SELECT DISTINCT {code} as code, {name} as name FROM arborescence_geo
    WHERE unaccent({name}) || {code} ILIKE unaccent(%s)
    AND "FR" <> 'ETR'
    ORDER BY {name}, {code}
    LIMIT {limit} OFFSET {offset};
    ;
    """).format(
        code=Identifier(mesh_db),
        name=Identifier(f"NOM_{mesh_db}"),
        offset=Literal(offset * pagination),
        limit=Literal(pagination),
    )
    territories = run_custom_query(query, [f"%{search}%"])
    lis = []
    for territory in territories:
        li = _fill_territory_li(territory["code"], territory["name"], mesh)
        lis.append(li)
    if len(territories) == pagination:
        li = f"""<li data-type="load"><button onclick="loadMoreTerritories(this)" data-mesh="{mesh}" data-offset="{offset + 1}" class="fr-btn fr-btn--secondary fr-btn--sm">Charger plus de résultats</button></li>"""
        lis.append(li)
    return HttpResponse("\n".join(lis))



import json
from datetime import timedelta

from django.http import HttpResponse, JsonResponse
from django.utils import timezone
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST
from pydantic import ValidationError

from territories_dashboard_lib.tracking_lib.logic import (
    track_event,
)
from territories_dashboard_lib.tracking_lib.models import Event
from territories_dashboard_lib.tracking_lib.payloads import EventPayload


@require_POST
@csrf_exempt
def track_event_view(request):
    try:
        data = json.loads(request.body)
        payload = EventPayload(**data)
    except json.JSONDecodeError:
        return JsonResponse({"error": "Invalid JSON"}, status=400)
    except ValidationError as e:
        return JsonResponse({"error": e.errors()}, status=422)
    if (
        Event.objects.filter(created_at__gte=timezone.now() - timedelta(days=1)).count()
        > 1000
    ):
        return HttpResponse(status=429)
    response = HttpResponse()
    track_event(
        request=request,
        response=response,
        event_name=payload.event,
        data={
            "indicator": payload.indicator,
            "objet": payload.objet,
            "type": payload.type,
        },
    )
    return HttpResponse()



import uuid
from datetime import timedelta
from typing import TYPE_CHECKING, Union

from django.http import HttpRequest, HttpResponse
from django.urls import resolve

from territories_dashboard_lib.tracking_lib.enums import TRACKING_COOKIE_NAME, EventType
from territories_dashboard_lib.tracking_lib.geolocalisation.logic import (
    get_client_ip,
    get_country_from_geolite,
)
from territories_dashboard_lib.tracking_lib.models import CookieInfo, Event, Page

if TYPE_CHECKING:
    from territories_dashboard_lib.indicators_lib.models import Indicator, Theme


def get_or_set_tracking_cookie(request: HttpRequest, response: HttpResponse) -> str:
    tracking_cookie = request.COOKIES.get(TRACKING_COOKIE_NAME)
    if tracking_cookie is None:
        tracking_cookie = str(uuid.uuid4())
        response.set_cookie(
            key=TRACKING_COOKIE_NAME,
            value=tracking_cookie,
            max_age=timedelta(days=365),
        )
    try:
        CookieInfo.objects.get(id=tracking_cookie)
    except CookieInfo.DoesNotExist:
        ip_address = get_client_ip(request)
        country = get_country_from_geolite(ip_address)
        CookieInfo.objects.create(
            id=tracking_cookie, country=country, ip_address=ip_address
        )
    return tracking_cookie


def track_page(
    *,
    request: HttpRequest,
    response: HttpResponse,
    params=None,
    indicator: Union["Indicator", None] = None,
    theme: Union["Theme", None] = None,
):
    if params is None:
        params = {}
    try:
        cookie = get_or_set_tracking_cookie(request, response)
        match = resolve(request.path)
        view_name = match.view_name
        Page.objects.create(
            cookie=cookie,
            territory_id=params.get("territory_id"),
            territory_mesh=params.get("territory_mesh"),
            submesh=params.get("mesh"),
            cmp_territory_id=params.get("cmp_territory_id"),
            cmp_territory_mesh=params.get("cmp_territory_mesh"),
            indicator=indicator.id if indicator else None,
            indicator_name=indicator.name if indicator else None,
            theme=theme.id if theme else None,
            theme_name=theme.name if theme else None,
            url=request.get_full_path(),
            view=view_name,
        )
    except Exception as e:
        print("error while tracking indicator view")
        print(e)
    return response


def track_event(
    *, request: HttpRequest, response: HttpResponse, event_name=EventType, data
):
    try:
        cookie = get_or_set_tracking_cookie(request, response)
        match = resolve(request.path)
        view_name = match.view_name
        Event.objects.create(
            cookie=cookie,
            name=event_name,
            data=data,
            url=request.get_full_path(),
            view=view_name,
        )
    except Exception as e:
        print("error while tracking event")
        print(e)
    return response



from django.db import models

TRACKING_COOKIE_NAME = "omnibus"


class EventType(models.TextChoices):
    download = "download"


class GraphType(models.TextChoices):
    comparaison_historique = "comparaison-historique"
    repartition_dimension = "repartition-dimension"
    repartition_valeurs = "repartition-valeurs"
    top_10 = "top_10"
    historique = "historique"
    comparison_histogram = "comparison-histogram"
    table = "table"



import os
from functools import lru_cache

import geoip2.database


def get_client_ip(request):
    x_forwarded_for = request.META.get("HTTP_X_FORWARDED_FOR")
    if x_forwarded_for:
        ip = x_forwarded_for.split(",")[0]
    else:
        ip = request.META.get("REMOTE_ADDR")
    return ip


@lru_cache(maxsize=100_000)
def get_country_from_geolite(ip_address):
    database_name = "GeoLite2-Country.mmdb"
    database_path = os.path.join(os.path.dirname(__file__), database_name)
    reader = geoip2.database.Reader(database_path)
    try:
        response = reader.country(ip_address)
        country = response.country.iso_code
    except geoip2.errors.AddressNotFoundError:
        country = "UNKNOWN"
    reader.close()
    return country
